Scriptname TweakPose extends ReferenceAlias  
{renn's Tweak ;TweakFollower}

; Dance can happen 2 different ways, which is why it doesn't just ride on the pose system.
int property currDance auto
Bool property isDancing auto

Int property statuePose auto
Bool property isPoseLoop auto
Bool property isStatue auto

Float property statuePosX auto
Float property statuePosY auto
Float property statuePosZ auto
Float property statueAngleX auto
Float property statueAngleY auto
Float property statueAngleZ auto
Int property command auto
Idle property DefaultSheathe auto

Faction Property TweakPosedFaction auto

; Deprecated 1.32
Scene Property poseScene  Auto 

TweakFollowerScript Property pTweakFollowerScript  Auto  

; Originally we didnt use a scene to keep the PC from moving thus some of this 
; code is for upgrade compatibility

Function AftReset()
	Actor npc = self.GetActorRef()
	npc.EnableAI(true)
	npc.SetUnconscious(FALSE)
	npc.GetActorBase().SetInvulnerable(FALSE)
	Debug.SendAnimationEvent( npc, "IdleForceDefaultState")
	(self as ReferenceAlias).UnregisterForAnimationEvent(self.getActorRef(), "weaponDraw")
	statuePose = 0
	statuePosX = 0.0
	statuePosY = 0.0
	statuePosZ = 0.0
	statueAngleX = 0.0
	statueAngleY = 0.0
	statueAngleZ = 0.0
	isStatue = FALSE
	isPoseLoop = FALSE
	isDancing = FALSE
	if (npc.IsInFaction(TweakPosedFaction) || poseScene.IsPlaying())
		if (poseScene.IsPlaying())
			poseScene.Stop()
		endif
		if (npc.IsInFaction(TweakPosedFaction))
			npc.RemoveFromFaction(TweakPosedFaction)
		endif
		Utility.wait(0.1)
		npc.ClearKeepOffsetFromActor()
	endif
	command=0
	currDance=0
endFunction

; One time operations
Function initialize()
	statuePose = 0
	statuePosX = 0.0
	statuePosY = 0.0
	statuePosZ = 0.0
	statueAngleX = 0.0
	statueAngleY = 0.0
	statueAngleZ = 0.0
	isStatue = FALSE
	isPoseLoop = FALSE
	isDancing = FALSE
	command = 0
	currDance = 0
	EventFollowingPlayer()
endFunction

; Every time they join the PC.
Function EventFollowingPlayer()

	Actor npc = self.GetActorRef()

	if (npc.isUnconscious())
		npc.SetUnconscious(FALSE)
	endif

	if (isStatue || poseScene.IsPlaying() || npc.IsInFaction(TweakPosedFaction))
		TweakSettings pTweakSettings = ((self as ReferenceAlias) As TweakSettings)
		npc.IgnoreFriendlyHits(pTweakSettings.originalIgnoreHits)
		npc.GetActorBase().SetInvulnerable(FALSE)

		; Stop any current animation
		isStatue = FALSE
		isDancing = FALSE
		isPoseLoop = FALSE
		if (poseScene.IsPlaying())
			poseScene.Stop()
		endif
		if (npc.IsInFaction(TweakPosedFaction))
			npc.RemoveFromFaction(TweakPosedFaction)
		endif
		Utility.wait(0.25)
		npc.ClearKeepOffsetFromActor()
		Debug.SendAnimationEvent( npc, "IdleForceDefaultState")
	endif
endFunction

; Every time they continue following the PC after waiting
Function EventWaitToFollow()
	; resolves a bug introduced by 1.3 (if they happen to be upgrading)
	Actor npc = self.GetActorRef()
	if (poseScene.IsPlaying() || npc.IsInFaction(TweakPosedFaction))
		TweakSettings pTweakSettings = ((self as ReferenceAlias) As TweakSettings)
		npc.IgnoreFriendlyHits(pTweakSettings.originalIgnoreHits)
		npc.GetActorBase().SetInvulnerable(FALSE)

		; Stop any current animation
		isStatue = FALSE
		isDancing = FALSE
		isPoseLoop = FALSE
		if (poseScene.IsPlaying())
			poseScene.Stop()
		endif
		if (npc.IsInFaction(TweakPosedFaction))
			npc.RemoveFromFaction(TweakPosedFaction)
		endif
		Utility.wait(0.25)
		npc.ClearKeepOffsetFromActor()
		Debug.SendAnimationEvent( npc, "IdleForceDefaultState")
	endif
EndFunction

; When they stop following the PC
Function EventNotFollowingPlayer()
	(self as ReferenceAlias).UnregisterForAnimationEvent(self.getActorRef(), "weaponDraw")

	; If they are not a statue, then stop current pose
	if (!isStatue && 0 != statuePose)
		statuePose = 0
		Debug.SendAnimationEvent( self.GetActorRef(), "IdleForceDefaultState")
	endif

endFunction

Event OnUpdate()

	Actor npc = self.GetActorRef()
	if !npc || npc.IsDead()
		return
	endIf

	; Freeze is done asynchronously from dialogue
	if (5 == command) ; Freeze
                ; If freeze comes before stopposing it effectively cancels the stopposing command

		npc.IgnoreFriendlyHits()
		npc.GetActorBase().SetInvulnerable()

		; npc.EnableAI(false)
		isStatue = true

		; Remember everything about the npc for onLoad restore
		; clothing, position, facing, pose

		statuePosX = npc.GetPositionX()
		statuePosY = npc.GetPositionY()
		statuePosZ = npc.GetPositionZ()

		statueAngleX = npc.GetAngleX()
		statueAngleY = npc.GetAngleY()
		statueAngleZ = npc.GetAngleZ()

		; deprecated 1.32
		; poseScene.Start()
		npc.AddToFaction(TweakPosedFaction)
		self.GetActorRef().KeepOffsetFromActor(npc,0,0,0,0,0,0,200,50)
		npc.EvaluatePackage()
		command = 24
		; Give dialogue time to disappear. It can take a while since we activates a scene
		RegisterForSingleUpdate(4.0)
		return
	elseif (24 == command) ; Stop posing
		PlayCurrentPose()
		command = 0
		return
	endif

	if (isDancing || isPoseLoop)
		if (isStatue)
			; Need to do this for deprecated reasons...
			npc.SetUnconscious(false)
		endif
		; If the PC leaves the area, stop the polling
		if (TRUE == npc.Is3DLoaded())
			if (isDancing)
				KeepDancing()
			else
				KeepPosing()
			endif
		endif
	elseif (isStatue)

		; Bug Fix. Sometimes NPCs get spammed with updates when told to wait in house. Dont know why.

		; Updates registered on both the Actor and the Alias will be recieved, but if
		; it was registered on the ACTOR, it must be unregsietered there as well or
		; unregister will not stop the event. 
		UnRegisterForUpdate()
		self.GetActorRef().UnRegisterForUpdate()
	endif

	if (23 == command) ; Stop posing
		; after 120 seconds (or the next event that generates an update), change pose back to none. 
		UnregisterForUpdate()
		isStatue = false
		statuePose = 0

		if (poseScene.IsPlaying() || npc.IsInFaction(TweakPosedFaction))
			if (poseScene.IsPlaying())
				poseScene.Stop()
			endif
			if (npc.IsInFaction(TweakPosedFaction))
				npc.RemoveFromFaction(TweakPosedFaction)
			endif
			Utility.wait(0.25)
			self.GetActorRef().ClearKeepOffsetFromActor()
		endif
		isPoseLoop = FALSE
		command = 0
	endif

endEvent

; Called when we enter the Pose menu to ensure no odd poses with weapons drawn.
; Also ensures PC can end the pose by drawing their weapon. 

Function PreparePose()
	Actor npc = self.GetActorRef()
	if (0 == statuePose)
		Game.GetPlayer().PlayIdle(DefaultSheathe)
		Self.GetActorRef().PlayIdle(DefaultSheathe)
	endif
	isDancing = false;
	isPoseLoop = false;
	if (poseScene.IsPlaying() || npc.IsInFaction(TweakPosedFaction))
		if (poseScene.IsPlaying())
			poseScene.Stop()
		endif
		if (npc.IsInFaction(TweakPosedFaction))
			npc.RemoveFromFaction(TweakPosedFaction)
		endif
		Utility.wait(0.25)
		self.GetActorRef().ClearKeepOffsetFromActor()
	endif
endFunction

Function PoseFollower(int pose)

	UnregisterForUpdate()
	Bool succ = (self as ReferenceAlias).RegisterForAnimationEvent(self.getActorRef(), "weaponDraw")
	if !succ
		int i = 8
		while (i > 0 && !succ)
			Utility.wait(0.25)
			succ = (self as ReferenceAlias).RegisterForAnimationEvent(self.getActorRef(), "weaponDraw")
			i -= 1
		EndWhile
	EndIf

	isDancing=false
	isPoseLoop = false;

	; NOTE: Update script assumes poses start at offest of 11. If more commands are
	; added, then update the TweakFollowerQuestScript with larger offset

	if (pose > 10)
		Debug.SendAnimationEvent( self.GetActorRef(), "IdleForceDefaultState")
		Utility.wait(0.25)
		if (11 == pose) ; Pose handsbehindback
			statuePose = 1
			Debug.SendAnimationEvent(self.GetActorRef(), "IdleGrave_02")
		elseif (12 == pose) ; Pose daydream
			statuePose = 2
			Debug.SendAnimationEvent(self.GetActorRef(), "IdleLaydown")
		elseif (13 == pose) ; Pose flex
			statuePose = 3
			Debug.SendAnimationEvent(self.GetActorRef(), "IdleStaticPoseAStart")
		elseif (14 == pose) ; Pose relax
			statuePose = 4
			Debug.SendAnimationEvent(self.GetActorRef(), "IdleWounded_01")
		elseif (15 == pose) ; Pose kneel
			statuePose = 5
			Debug.SendAnimationEvent(self.GetActorRef(), "IdleKneelingEnter")
		elseif (16 == pose) ; Pose bound
			statuePose = 6
			Debug.SendAnimationEvent(self.GetActorRef(), "IdleBoundKneesEnterInstant")
		elseif (17 == pose) ; Pose afraid
			statuePose = 7
			Debug.SendAnimationEvent(self.GetActorRef(), "IdleCowering")
		elseif (18 == pose) ; Pose handsinfront
			statuePose = 8
			Debug.SendAnimationEvent(self.GetActorRef(), "IdleGrave_01")
		elseif (19 == pose) ; Pose pray
			statuePose = 9
			Debug.SendAnimationEvent(self.GetActorRef(), "IdlePrayCrouchedEnter")
		elseif (20 == pose) ; Pose meditate
			statuePose = 10
			Debug.SendAnimationEvent(self.GetActorRef(), "IdleGreybeardMeditateEnter")
		elseif (21 == pose) ; Pose warmhands
			statuePose = 11
			Debug.SendAnimationEvent(self.GetActorRef(), "IdleWarmHandsCrouched")
		elseif (22 == pose) ; Pose lie down
			statuePose = 12
			Debug.SendAnimationEvent(self.GetActorRef(), "IdleAstridEnter")
		elseif (28 == pose) ; Pose Sit Ground / renn
			statuePose = 18
			Debug.SendAnimationEvent(self.GetActorRef(), "IdleSitCrossLeggedEnter")
		elseif (pose > 22) && (pose < 28)
			isDancing=true
			Actor npc = self.GetActorRef()
			statuePosX = npc.GetPositionX()
			statuePosY = npc.GetPositionY()
			statuePosZ = npc.GetPositionZ()
			if (23 == pose) ; Dance - Normal
				statuePose = 13
				currDance = 3
			elseif (24 == pose) ; Dance - Belly 
				statuePose = 14
				currDance = 1
			elseif (25 == pose) ; Dance - Snake 
				statuePose = 15
				currDance = 2
			elseif (26 == pose) ; Dance - BumpandGrind
				statuePose = 16
				currDance = 5
			elseif (27 == pose) ; Dance - BumpandGrind
				statuePose = 17
				currDance = 4
			endif

			KeepDancing()
			return
		endif
	endif
	command = 23
	RegisterForSingleUpdate(120)

endFunction

; This is actually a hack for detecting when the npc draws their weapon. 
Event OnAnimationEvent(ObjectReference akSource, string asEventName)
	Actor npc = self.GetActorRef()
	if !npc || npc.IsDead()
		return
	endIf

	if (0 != statuePose && "weaponDraw" == asEventName)
		if (self.getActorRef().GetFormID() == akSource.GetFormID())
			; Note the scope of the event registration is the actor.
			; So we are not competing with Tweak Settings which monitors
			; the player. 
			(self as ReferenceAlias).UnregisterForAnimationEvent(self.getActorRef(), "weaponDraw")
			if (!isStatue)
				statuePose = 0
				if (poseScene.IsPlaying() || self.getActorRef().IsInFaction(TweakPosedFaction))
					if (poseScene.IsPlaying())
						poseScene.Stop()
					endif
					if (self.getActorRef().IsInFaction(TweakPosedFaction))
						self.getActorRef().RemoveFromFaction(TweakPosedFaction)
					endif
					Utility.wait(0.25)
					self.GetActorRef().ClearKeepOffsetFromActor()
				endif
			endif
		endif
	endif
endEvent

;Event OnUnLoad()
;	if (isStatue)
;		if (isDancing || isPoseLoop)
;			Actor npc = self.GetActorRef()
;			npc.SetUnconscious(false);
;			Debug.SendAnimationEvent( npc, "IdleForceDefaultState")
;		endif
;	endif
;endEvent

Event OnLoad()
	Actor npc = self.GetActorRef()
	if !npc || npc.IsDead()
		return
	endIf

	if (isStatue)
		int max = 8
		While(FALSE == npc.Is3DLoaded() && max != 0)
			Utility.wait(0.25)
			max -= 1
		EndWhile
		if (npc.IsInFaction(TweakPosedFaction))
			npc.KeepOffsetFromActor(npc,0,0,0,0,0,0,200,50)
		EndIf
	EndIf
EndEvent

Event OnCellLoad()

	Actor npc = self.GetActorRef()
	if !npc || npc.IsDead()
		return
	endIf

	if (isStatue)
		if (isDancing || isPoseLoop)
			npc.SetUnconscious(false);
;			Debug.SendAnimationEvent( npc, "IdleForceDefaultState")
		endif
		; delay needed for cell to be ready to manipulate animations.
		Utility.wait(0.25)
		
		int max = 8
		While(FALSE == npc.Is3DLoaded() && max != 0)
			Utility.wait(0.25)
			max -= 1
		EndWhile

		if (npc.IsInFaction(TweakPosedFaction))
			npc.KeepOffsetFromActor(npc,0,0,0,0,0,0,200,50)
		else
			; This is a sign of an upgrade from an old version
			if (!poseScene.IsPlaying())
				; a really old version...
				npc.SetUnconscious(true);
				npc.EnableAI(true)
				Utility.wait(1.00)
				npc.SetUnconscious(false);
			endif

			npc.AddToFaction(TweakPosedFaction)
			npc.KeepOffsetFromActor(npc,0,0,0,0,0,0,200,50)

			if (poseScene.IsPlaying())
				npc.SetUnconscious(false);
				Utility.wait(0.50) 
				poseScene.Stop()
			endif
			npc.EvaluatePackage()
		endif

		if (isDancing)
			KeepDancing()
		elseif (isPoseLoop)
			KeepPosing()
		else
			PlayCurrentPose()
			Utility.wait(2.0) ; Allow time to get into pose
			npc.SetUnconscious(false);
			Utility.wait(0.50)
			PlayCurrentPose()
		endif

		npc.GetActorBase().SetInvulnerable()
		npc.IgnoreFriendlyHits()
	endif

endEvent

Event OnActivate(ObjectReference akActionRef)

	Actor npc = self.GetActorRef()
	if !npc || npc.IsDead()
		return
	endIf

	if (isStatue)
		; allow them to talk
		Debug.SendAnimationEvent( npc, "IdleForceDefaultState")
		if (npc.isUnconscious())
			npc.SetUnconscious(FALSE)
		endif
		if (!npc.IsInFaction(TweakPosedFaction))
			if (poseScene.IsPlaying())
				; Upgraded from 1.31
				npc.AddToFaction(TweakPosedFaction)
				npc.KeepOffsetFromActor(npc,0,0,0,0,0,0,200,50)
				poseScene.Stop()
				npc.EvaluatePackage()
			else
				; Upgraded from 1.21
				TweakSettings pTweakSettings = ((self as ReferenceAlias) As TweakSettings)
				npc.EnableAI(true)
				npc.IgnoreFriendlyHits(pTweakSettings.originalIgnoreHits)
				npc.GetActorBase().SetInvulnerable(FALSE)
				isStatue = FALSE
			endif
		endif
	endif

endEvent

Function PlayCurrentPose()

	if (0 == statuePose) 
		return
	endif

	isDancing=false
	isPoseLoop=false

	Actor npc = self.GetActorRef()
	statuePosX = npc.GetPositionX()
	statuePosY = npc.GetPositionY()
	statuePosZ = npc.GetPositionZ()

	; Stop any current animation
	; Utility.wait(0.1)

	if (1 == statuePose) ; handsbehindback
		isPoseLoop = true
		KeepPosing()
	elseif (2 == statuePose) ; daydream
		isPoseLoop = true
		KeepPosing()
	elseif (3 == statuePose) ; Pose flex
		Debug.SendAnimationEvent(self.GetActorRef(), "IdleStaticPoseAStart")
	elseif (4 == statuePose) ; Pose relax
		Debug.SendAnimationEvent(self.GetActorRef(), "IdleWounded_01")
	elseif (5 == statuePose) ; Pose kneel
		Debug.SendAnimationEvent(self.GetActorRef(), "IdleKneelingEnter")
	elseif (6 == statuePose) ; bound
		Debug.SendAnimationEvent(self.GetActorRef(), "IdleBoundKneesEnterInstant")
	elseif (7 == statuePose) ; afraid
		isPoseLoop = true
		KeepPosing()
	elseif (8 == statuePose) ; handsinfront
		isPoseLoop = true
		KeepPosing()
	elseif (9 == statuePose) ; Pose pray
		Debug.SendAnimationEvent(self.GetActorRef(), "IdlePrayCrouchedEnter")
	elseif (10 == statuePose) ; Pose meditate
		Debug.SendAnimationEvent(self.GetActorRef(), "IdleGreybeardMeditateEnter")
	elseif (11 == statuePose) ; warmhands
		isPoseLoop = true
		KeepPosing()
	elseif (12 == statuePose) ; lie down
		Debug.SendAnimationEvent(self.GetActorRef(), "IdleAstridEnter")
	elseif (18 == statuePose) ; Sit Ground / renn
		isPoseLoop = true
		KeepPosing()
	elseif (statuePose > 12) && (statuePose < 18); 

		if (13 == statuePose) ; Normal
			currDance = 3
		elseif (14 == statuePose) ; Belly
			currDance = 1
		elseif (15 == statuePose) ; Snake
			currDance = 2
		elseif (16 == statuePose) ; Bump and Grind
			currDance = 5
		elseif (17 == statuePose) ; hip hop
			currDance = 4
		endif

		isDancing=true
		KeepDancing()
	endif

endFunction

Function Dance()
	isDancing=true
	Actor npc = self.GetActorRef()
	statuePosX = npc.GetPositionX()
	statuePosY = npc.GetPositionY()
	statuePosZ = npc.GetPositionZ()
	currDance += 1
	if (currDance > 5)
		currDance = 1
	endif
	if (1 == currDance)
		Debug.Notification("Dance: Belly")
	elseif (2 == currDance)
		Debug.Notification("Dance: Snake")
	elseif (3 == currDance)
		Debug.Notification("Dance: Normal")
	elseif (4 == currDance)
		Debug.Notification("Dance: Hip Hop")
	elseif (5 == currDance)
		Debug.Notification("Dance: Bump and Grind")
	endIf

	KeepDancing()
endFunction

Function KeepPosing()
	Actor npc = self.GetActorRef()
	Actor pc = Game.GetPlayer()
	if (WithinRadius(statuePosX, statuePosY, statuePosZ, npc.GetPositionX(), npc.GetPositionY(), npc.GetPositionZ(), 60))
;		if (isStatue)
			; We keep them unconscious so the Pose Scene AI package wont trump it. 
;			npc.SetUnconscious(true);
;		endif
		if (1 == statuePose) ; handsbehindback
			Debug.SendAnimationEvent(self.GetActorRef(), "IdleGrave_02")
			RegisterForSingleUpdate(15.0)
		elseif (2 == statuePose) ; daydream
			Debug.SendAnimationEvent(self.GetActorRef(), "IdleLaydown")
			RegisterForSingleUpdate(15.0)
		elseif (3 == statuePose) ; flex
			Debug.SendAnimationEvent(self.GetActorRef(), "IdleStaticPoseAStart")
			RegisterForSingleUpdate(15.0)
		elseif (7 == statuePose) ; afraid
			Debug.SendAnimationEvent(self.GetActorRef(), "IdleCowering")
			RegisterForSingleUpdate(15.0)
		elseif (8 == statuePose) ; handsinfront
			Debug.SendAnimationEvent(self.GetActorRef(), "IdleGrave_01")
			RegisterForSingleUpdate(15.0)
		elseif (18 == statuePose) ; Sit Ground / renn
			Debug.SendAnimationEvent(self.GetActorRef(), "IdleSitCrossLeggedEnter")
			RegisterForSingleUpdate(15.0)
		else; (11 == statuePose) ; warmhands
			Debug.SendAnimationEvent(self.GetActorRef(), "IdleWarmHandsCrouched")
			RegisterForSingleUpdate(15.0)
		endIf
	else
		isPoseLoop = false
	endif
endFunction

Function KeepDancing()
	Actor npc = self.GetActorRef()
	Actor pc = Game.GetPlayer()
	if (WithinRadius(statuePosX, statuePosY, statuePosZ, npc.GetPositionX(), npc.GetPositionY(), npc.GetPositionZ(), 60))
;		if (isStatue)
			; We keep them unconscious so the Pose Scene AI package wont trump it. 
;			npc.SetUnconscious(true);
;		endif
		if (1 == currDance)
			Debug.SendAnimationEvent(npc, "IdleCiceroDance1")
			RegisterForSingleUpdate(19)
		elseif (2 == currDance)
			Debug.SendAnimationEvent(npc, "IdleCiceroDance2")
			RegisterForSingleUpdate(19)
		elseif (3 == currDance)
			Debug.SendAnimationEvent(npc, "IdleCiceroDance3")
			RegisterForSingleUpdate(38)
		elseif (4 == currDance)
			Debug.SendAnimationEvent(npc, "IdleCiceroHappy")
			RegisterForSingleUpdate(19)
		else; 5 ==  currDance
			Debug.SendAnimationEvent(npc, "IdleCiceroAgitated")
			RegisterForSingleUpdate(6.5)
		endIf
	else
		isDancing = false
	endif
endFunction

; Quickly Calculate if 2 points are within radius in 3D:
; Distance = SquareRoot((x2-x1)^2 + (y2-y1)^2 + (z2-z1)^2)
; ** If you dont need precise distance, you can avoid the innefficient square root function
; ** Not a good general function. Every cell has a different coordinate system. Use this only to 
;      detect if NPC has moved from an originally stored location.
;  if (Distance)^2 > (x2-x1)^2 + (y2-y1)^2 + (z2-z1)^2

Bool Function WithinRadius(float a_x, float a_y, float a_z, float b_x, float b_y, float b_z, int radius=300)

    float xd = (a_x - b_x)
    float yd = (a_y - b_y)
    float zd = (a_z - b_z)
    return (radius * radius) > ( xd * xd ) + ( yd * yd ) + ( zd * zd )

endFunction